@@ -255,7 +255,7 @@ sendDirectoryDocument(TList *      const listP,
         }
 
         ftm = *gmtime(&fi->time_write);
-        sprintf(z2, "%02u/%02u/%04u %02u:%02u:%02u",ftm.tm_mday,ftm.tm_mon,
+        sprintf(z2, "%02u/%02u/%04u %02u:%02u:%02u",ftm.tm_mday,ftm.tm_mon+1,
                 ftm.tm_year+1900,ftm.tm_hour,ftm.tm_min,ftm.tm_sec);
 
         if (fi->attrib & A_SUBDIR) {
@@ -1118,6 +1118,8 @@ serverFunc(void * const userHandle) {
 
         if (!success)
             connectionDone = TRUE;
+        else if (srvP->terminationRequested)
+            connectionDone = TRUE;
         else {
             abyss_bool const lastReqOnConn =
                 requestCount + 1 >= srvP->keepalivemaxconn;
@@ -1732,7 +1734,8 @@ void
 ServerDefaultHandler(TServer *  const serverP,
                      URIHandler const handler) {
 
-    serverP->srvP->defaulthandler = handler;
+    serverP->srvP->defaulthandler =
+        handler ? handler : ServerDefaultHandlerFunc;
 }
 
 
