@@ -56,6 +56,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <assert.h>
+#include <sys/time.h>
 
 #include "xmlrpc_config.h"
 
@@ -299,7 +300,7 @@ curlMulti_perform(xmlrpc_env *       const envP,
         if (rc != CURLM_OK) {
             xmlrpc_faultf(envP,
                           "Impossible failure of curl_multi_perform() "
-                          "with rc %d (%s)", rc, curl_multi_strerror(rc));
+                          "with rc %d", rc);
         }
     }
 }        
@@ -322,8 +323,7 @@ curlMulti_addHandle(xmlrpc_env *       const envP,
     if (rc != CURLM_OK)
         xmlrpc_faultf(envP, "Could not add Curl session to the "
                       "curl multi manager.  curl_multi_add_handle() "
-                      "returns error code %d (%s)",
-                      rc, curl_multi_strerror(rc));
+                      "returns error code %d", rc);
 }
 
 
@@ -424,7 +424,7 @@ curlMulti_fdset(xmlrpc_env *       const envP,
 
     if (rc != CURLM_OK)
         xmlrpc_faultf(envP, "Impossible failure of curl_multi_fdset() "
-                      "with rc %d (%s)", rc, curl_multi_strerror(rc));
+                      "with rc %d", rc);
 }
 
 
@@ -1214,7 +1214,8 @@ setupCurlSession(xmlrpc_env *             const envP,
             curl_easy_setopt(curlSessionP, CURLOPT_SSLENGINE,
                              curlSetupP->sslEngine);
         if (curlSetupP->sslEngineDefault)
-            curl_easy_setopt(curlSessionP, CURLOPT_SSLENGINE_DEFAULT);
+            /* 3rd argument seems to be required by some Curl */
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLENGINE_DEFAULT, 1l);
         if (curlSetupP->sslVersion != XMLRPC_SSLVERSION_DEFAULT)
             curl_easy_setopt(curlSessionP, CURLOPT_SSLVERSION,
                              curlSetupP->sslVersion);
@@ -1489,7 +1490,8 @@ sendRequest(xmlrpc_env *                     const envP,
 
 static void
 finishCurlTransaction(xmlrpc_env * const envP ATTR_UNUSED,
-                      CURL *       const curlSessionP) {
+                      CURL *       const curlSessionP,
+                      CURLcode     const result) {
 /*----------------------------------------------------------------------------
   Handle the event that a Curl transaction has completed on the Curl
   session identified by 'curlSessionP'.
@@ -1514,7 +1516,12 @@ finishCurlTransaction(xmlrpc_env * const envP ATTR_UNUSED,
 
         xmlrpc_env_init(&env);
 
-        getCurlTransactionError(curlTransactionP, &env);
+        if (result != CURLE_OK) {
+            xmlrpc_env_set_fault_formatted(
+                envP, XMLRPC_NETWORK_ERROR, "libcurl failed to execute the "
+                "HTTP POST transaction.  %s", curlTransactionP->curlError);
+        } else
+            getCurlTransactionError(curlTransactionP, &env);
 
         rpcP->complete(rpcP->callInfoP, rpcP->responseXmlP, env);
 
@@ -1585,7 +1592,8 @@ processCurlMessages(xmlrpc_env *       const envP,
 
         if (!endOfMessages) {
             if (curlMsg.msg == CURLMSG_DONE)
-                finishCurlTransaction(envP, curlMsg.easy_handle);
+                finishCurlTransaction(envP, curlMsg.easy_handle,
+                                      curlMsg.data.result);
         }
     }
 }
