@@ -5,6 +5,7 @@
 #include <sstream>
 #include <memory>
 #include <time.h>
+#include <cstring>
 
 #include "xmlrpc-c/girerr.hpp"
 using girerr::error;
@@ -14,6 +15,7 @@ using girerr::error;
 #include "xmlrpc-c/registry.hpp"
 
 #include "testclient.hpp"
+#include "server_abyss.hpp"
 #include "tools.hpp"
 
 using namespace xmlrpc_c;
@@ -50,7 +52,7 @@ using namespace std;
 class sampleAddMethod : public method {
 public:
     sampleAddMethod() {
-        this->_signature = "ii";
+        this->_signature = "i:ii";
         this->_help = "This method adds two integers together";
     }
     void
@@ -254,11 +256,6 @@ void test_value (void) {
     }
 }
 
-void test_errors (void) {
-    // XXX - Test typechecks on get* methods.
-    // XXX - Test typechceks on array and struct methods.
-    // XXX - Test bounds checks on arrayGetItem, structGetKeyAndValue.
-}
 
 
 static void
@@ -271,7 +268,6 @@ testXmlRpcCpp() {
     test_fault();
     test_env();
     test_value();
-    test_errors();
 }
 
 
@@ -794,6 +790,7 @@ main(int argc, char**) {
         valueTestSuite().run(0);
         paramListTestSuite().run(0);
         registryTestSuite().run(0);
+        serverAbyssTestSuite().run(0);
         clientTestSuite().run(0);
 
         testXmlRpcCpp();
