@@ -28,6 +28,195 @@ using girerr::error;
 using namespace xmlrpc_c;
 using namespace std;
 
+
+
+class sampleAddMethod : public method {
+public:
+    sampleAddMethod() {
+        this->_signature = "i:ii";
+        this->_help = "This method adds two integers together";
+    }
+    void
+    execute(xmlrpc_c::paramList const& paramList,
+            value *             const  retvalP) {
+        
+        int const addend(paramList.getInt(0));
+        int const adder(paramList.getInt(1));
+        
+        paramList.verifyEnd(2);
+        
+        *retvalP = value_int(addend + adder);
+    }
+};
+
+
+
+class carriageParm_direct : public carriageParm {
+public:
+    carriageParm_direct(registry * const registryP) : registryP(registryP) {}
+
+    registry * registryP;
+};
+
+
+class clientXmlTransport_direct : public clientXmlTransport {
+
+public:    
+    void
+    call(xmlrpc_c::carriageParm * const  carriageParmP,
+         string                   const& callXml,
+         string *                 const  responseXmlP) {
+
+        carriageParm_direct * const parmP =
+            dynamic_cast<carriageParm_direct *>(carriageParmP);
+
+        if (parmP == NULL)
+            throw(error("Carriage parameter passed to the direct "
+                        "transport is not type carriageParm_direct"));
+
+        parmP->registryP->processCall(callXml, responseXmlP);
+    }
+};
+
+
+
+class clientDirectAsyncTestSuite : public testSuite {
+/*----------------------------------------------------------------------------
+   See clientDirectTestSuite for a description of how we use a
+   clientXmlTransport_direct object to test client functions.
+
+   The object of this class tests the async client functions.  With
+   clientXmlTransport_direct, these are pretty simple because the
+   transport doesn't even implement an asynchronous interface; it
+   relies on the base class' emulation of start() using call().
+
+   Some day, we should add true asynchronous capability to
+   clientXmlTransport_direct and really test things.
+-----------------------------------------------------------------------------*/
+public:
+    virtual string suiteName() {
+        return "clientDirectAsyncTestSuite";
+    }
+    virtual void runtests(unsigned int const) {
+        
+        registry myRegistry;
+        
+        myRegistry.addMethod("sample.add", methodPtr(new sampleAddMethod));
+        
+        carriageParm_direct carriageParmDirect(&myRegistry);
+        clientXmlTransport_direct transportDirect;
+        client_xml clientDirect(&transportDirect);
+        paramList paramListSampleAdd1;
+        paramListSampleAdd1.add(value_int(5));
+        paramListSampleAdd1.add(value_int(7));
+        paramList paramListSampleAdd2;
+        paramListSampleAdd2.add(value_int(30));
+        paramListSampleAdd2.add(value_int(-10));
+
+        rpcPtr const rpcSampleAdd1P("sample.add", paramListSampleAdd1);
+        rpcSampleAdd1P->start(&clientDirect, &carriageParmDirect);
+        rpcPtr const rpcSampleAdd2P("sample.add", paramListSampleAdd2);
+        rpcSampleAdd2P->start(&clientDirect, &carriageParmDirect);
+
+        TEST(rpcSampleAdd1P->isFinished());
+        TEST(rpcSampleAdd1P->isSuccessful());
+        value_int const result1(rpcSampleAdd1P->getResult());
+        TEST(static_cast<int>(result1) == 12);
+        
+        TEST(rpcSampleAdd2P->isFinished());
+        TEST(rpcSampleAdd1P->isSuccessful());
+        value_int const result2(rpcSampleAdd2P->getResult());
+        TEST(static_cast<int>(result2) == 20);
+
+        EXPECT_ERROR(clientDirect.finishAsync(timeout()););
+        EXPECT_ERROR(clientDirect.finishAsync(timeout(50)););
+    }
+};
+
+
+
+class clientDirectTestSuite : public testSuite {
+/*----------------------------------------------------------------------------
+  The object of this class tests the client facilities by using a
+  special client XML transport defined above and an XML-RPC server we
+  build ourselves and run inline.  We build the server out of a
+  xmlrpc_c::registry object and our transport just delivers XML
+  directly to the registry object and gets the response XML from it
+  and delivers that back.  There's no network or socket or pipeline or
+  anything -- the transport actually executes the XML-RPC method.
+-----------------------------------------------------------------------------*/
+public:
+    virtual string suiteName() {
+        return "clientDirectTestSuite";
+    }
+    virtual void runtests(unsigned int const indentation) {
+        registry myRegistry;
+        
+        myRegistry.addMethod("sample.add", methodPtr(new sampleAddMethod));
+        
+        carriageParm_direct carriageParmDirect(&myRegistry);
+        clientXmlTransport_direct transportDirect;
+        client_xml clientDirect(&transportDirect);
+        paramList paramListSampleAdd;
+        paramListSampleAdd.add(value_int(5));
+        paramListSampleAdd.add(value_int(7));
+        paramList paramListEmpty;
+        {
+            /* Test a successful RPC */
+            rpcPtr rpcSampleAddP("sample.add", paramListSampleAdd);
+            rpcSampleAddP->call(&clientDirect, &carriageParmDirect);
+            TEST(rpcSampleAddP->isFinished());
+            TEST(rpcSampleAddP->isSuccessful());
+            value_int const resultDirect(rpcSampleAddP->getResult());
+            TEST(static_cast<int>(resultDirect) == 12);
+        }
+        {
+            /* Test a failed RPC */
+            rpcPtr const rpcSampleAddP("sample.add", paramListEmpty);
+            rpcSampleAddP->call(&clientDirect, &carriageParmDirect);
+            TEST(rpcSampleAddP->isFinished());
+            TEST(!rpcSampleAddP->isSuccessful());
+            fault const fault0(rpcSampleAddP->getFault());
+            TEST(fault0.getCode() == fault::CODE_TYPE);
+        }
+
+        {
+            /* Test with an auto object transport */
+            client_xml clientDirect(
+                clientXmlTransportPtr(new clientXmlTransport_direct));
+            rpcPtr rpcSampleAddP("sample.add", paramListSampleAdd);
+            rpcSampleAddP->call(&clientDirect, &carriageParmDirect);
+            TEST(rpcSampleAddP->isFinished());
+            TEST(rpcSampleAddP->isSuccessful());
+            EXPECT_ERROR(fault fault0(rpcSampleAddP->getFault()););
+            value_int const resultDirect(rpcSampleAddP->getResult());
+            TEST(static_cast<int>(resultDirect) == 12);
+        }
+        {
+            /* Test with implicit RPC -- success */
+            rpcOutcome outcome;
+            clientDirect.call(&carriageParmDirect, "sample.add",
+                              paramListSampleAdd, &outcome);
+            TEST(outcome.succeeded());
+            value_int const result(outcome.getResult());
+            TEST(static_cast<int>(result) == 12);
+        }
+        {
+            /* Test with implicit RPC - failure */
+            rpcOutcome outcome;
+            clientDirect.call(&carriageParmDirect, "nosuchmethod",
+                              paramList(), &outcome);
+            TEST(!outcome.succeeded());
+            TEST(outcome.getFault().getCode() == fault::CODE_NO_SUCH_METHOD);
+            TEST(outcome.getFault().getDescription().size() > 0);
+        }
+
+        clientDirectAsyncTestSuite().run(indentation+1);
+    }
+};
+
+
+
 class curlTransportTestSuite : public testSuite {
 
 public:
@@ -129,6 +318,35 @@ public:
 
 
 
+class ambivalentTransportTestSuite : public testSuite {
+
+public:
+    virtual string suiteName() {
+        return "ambivalentTransportTestSuite";
+    }
+    virtual void runtests(unsigned int const) {
+        vector<string> const typeList(
+            clientXmlTransport_http::availableTypes());
+
+        TEST(typeList.size() > 0);
+
+        clientXmlTransportPtr const transportP(
+            clientXmlTransport_http::create());
+        carriageParm_http0 carriageParm0("http://whatsamatta.edux");
+        client_xml client0(transportP);
+        
+        rpcOutcome outcome;
+
+        // Fails because there's no such server
+        EXPECT_ERROR(
+            client0.call(&carriageParm0, "nosuchmethod", paramList(),
+                         &outcome);
+            );
+    }
+};
+
+
+
 class clientXmlTransportTestSuite : public testSuite {
 
 public:
@@ -139,6 +357,7 @@ public:
         curlTransportTestSuite().run(indentation + 1);
         libwwwTransportTestSuite().run(indentation + 1);
         wininetTransportTestSuite().run(indentation + 1);
+        ambivalentTransportTestSuite().run(indentation + 1);
     }
 };
 
@@ -172,190 +391,55 @@ public:
         
 
 
-class carriageParm_direct : public carriageParm {
-public:
-    carriageParm_direct(registry * const registryP) : registryP(registryP) {}
-
-    registry * registryP;
-};
-
-
-class clientXmlTransport_direct : public clientXmlTransport {
-
-public:    
-    void
-    call(xmlrpc_c::carriageParm * const  carriageParmP,
-         string                   const& callXml,
-         string *                 const  responseXmlP) {
-
-        carriageParm_direct * const parmP =
-            dynamic_cast<carriageParm_direct *>(carriageParmP);
-
-        if (parmP == NULL)
-            throw(error("Carriage parameter passed to the direct "
-                        "transport is not type carriageParm_direct"));
-
-        parmP->registryP->processCall(callXml, responseXmlP);
-    }
-};
-
-
-
-class sampleAddMethod : public method {
-public:
-    sampleAddMethod() {
-        this->_signature = "ii";
-        this->_help = "This method adds two integers together";
-    }
-    void
-    execute(xmlrpc_c::paramList const& paramList,
-            value *             const  retvalP) {
-        
-        int const addend(paramList.getInt(0));
-        int const adder(paramList.getInt(1));
-        
-        paramList.verifyEnd(2);
-        
-        *retvalP = value_int(addend + adder);
-    }
-};
-
-
-
-class clientDirectAsyncTestSuite : public testSuite {
+class clientCurlTestSuite : public testSuite {
 /*----------------------------------------------------------------------------
-   See clientDirectTestSuite for a description of how we use a
-   clientXmlTransport_direct object to test client functions.
+  The object of this class tests the combination of a client with
+  Curl transport.  We assume Curl transports themselves have already
+  been tested and clients with direct transports have already been tested.
 
-   The object of this class tests the async client functions.  With
-   clientXmlTransport_direct, these are pretty simple because the
-   transport doesn't even implement an asynchronous interface; it
-   relies on the base class' emulation of start() using call().
+  We don't have an HTTP server, so we test only superficially.
 
-   Some day, we should add true asynchronous capability to
-   clientXmlTransport_direct and really test things.
+  In the future, we could either start a server or use some server that's
+  normally avaailble on the Internet.
 -----------------------------------------------------------------------------*/
 public:
     virtual string suiteName() {
-        return "clientDirectAsyncTestSuite";
+        return "clientCurlTestSuite";
     }
-    virtual void runtests(unsigned int const) {
+    virtual void runtests(unsigned int) {
+#if MUST_BUILD_CURL_CLIENT
+        clientXmlTransport_curl transportc0;
+        client_xml client0(&transportc0);
+        carriageParm_http0 carriageParmHttp("http://suckthis.com");
+        carriageParm_curl0 carriageParmCurl("http://suckthis.com");
+        connection connection0(&client0, &carriageParmHttp);
+        paramList paramList0;
         
-        registry myRegistry;
-        
-        myRegistry.addMethod("sample.add", methodPtr(new sampleAddMethod));
-        
-        carriageParm_direct carriageParmDirect(&myRegistry);
-        clientXmlTransport_direct transportDirect;
-        client_xml clientDirect(&transportDirect);
-        paramList paramListSampleAdd1;
-        paramListSampleAdd1.add(value_int(5));
-        paramListSampleAdd1.add(value_int(7));
-        paramList paramListSampleAdd2;
-        paramListSampleAdd2.add(value_int(30));
-        paramListSampleAdd2.add(value_int(-10));
+        rpcOutcome outcome0;
 
-        rpcPtr const rpcSampleAdd1P("sample.add", paramListSampleAdd1);
-        rpcSampleAdd1P->start(&clientDirect, &carriageParmDirect);
-        rpcPtr const rpcSampleAdd2P("sample.add", paramListSampleAdd2);
-        rpcSampleAdd2P->start(&clientDirect, &carriageParmDirect);
-        
-        TEST(rpcSampleAdd1P->isFinished());
-        TEST(rpcSampleAdd1P->isSuccessful());
-        value_int const result1(rpcSampleAdd1P->getResult());
-        TEST(static_cast<int>(result1) == 12);
-        
-        TEST(rpcSampleAdd2P->isFinished());
-        TEST(rpcSampleAdd1P->isSuccessful());
-        value_int const result2(rpcSampleAdd2P->getResult());
-        TEST(static_cast<int>(result2) == 20);
+        // This fails because server doesn't exist
+        EXPECT_ERROR(
+            client0.call(&carriageParmHttp, "blowme", paramList0, &outcome0);
+                );
 
-        EXPECT_ERROR(clientDirect.finishAsync(timeout()););
-        EXPECT_ERROR(clientDirect.finishAsync(timeout(50)););
-    }
-};
+        // This fails because server doesn't exist
+        EXPECT_ERROR(
+            client0.call(&carriageParmCurl, "blowme", paramList0, &outcome0);
+            );
 
+        rpcPtr rpc0P("blowme", paramList0);
 
+        // This fails because server doesn't exist
+        EXPECT_ERROR(rpc0P->call(&client0, &carriageParmCurl););
 
-class clientDirectTestSuite : public testSuite {
-/*----------------------------------------------------------------------------
-  The object of this class tests the client facilities by using a
-  special client XML transport defined above and an XML-RPC server we
-  build ourselves and run inline.  We build the server out of a
-  xmlrpc_c::registry object and our transport just delivers XML
-  directly to the registry object and gets the response XML from it
-  and delivers that back.  There's no network or socket or pipeline or
-  anything -- the transport actually executes the XML-RPC method.
------------------------------------------------------------------------------*/
-public:
-    virtual string suiteName() {
-        return "clientDirectTestSuite";
-    }
-    virtual void runtests(unsigned int const indentation) {
-        registry myRegistry;
-        
-        myRegistry.addMethod("sample.add", methodPtr(new sampleAddMethod));
-        
-        carriageParm_direct carriageParmDirect(&myRegistry);
-        clientXmlTransport_direct transportDirect;
-        client_xml clientDirect(&transportDirect);
-        paramList paramListSampleAdd;
-        paramListSampleAdd.add(value_int(5));
-        paramListSampleAdd.add(value_int(7));
-        paramList paramListEmpty;
-        {
-            /* Test a successful RPC */
-            rpcPtr rpcSampleAddP("sample.add", paramListSampleAdd);
-            rpcSampleAddP->call(&clientDirect, &carriageParmDirect);
-            TEST(rpcSampleAddP->isFinished());
-            TEST(rpcSampleAddP->isSuccessful());
-            EXPECT_ERROR(fault fault0(rpcSampleAddP->getFault()););
-            value_int const resultDirect(rpcSampleAddP->getResult());
-            TEST(static_cast<int>(resultDirect) == 12);
-        }
-        {
-            /* Test a failed RPC */
-            rpcPtr const rpcSampleAddP("sample.add", paramListEmpty);
-            rpcSampleAddP->call(&clientDirect, &carriageParmDirect);
-            TEST(rpcSampleAddP->isFinished());
-            TEST(!rpcSampleAddP->isSuccessful());
-            EXPECT_ERROR(value result(rpcSampleAddP->getResult()););
-            fault const fault0(rpcSampleAddP->getFault());
-            TEST(fault0.getCode() == fault::CODE_TYPE);
-        }
+        rpcPtr rpc1P("blowme", paramList0);
+        // This fails because server doesn't exist
+        EXPECT_ERROR(rpc1P->call(connection0););
 
-        {
-            /* Test with an auto object transport */
-            client_xml clientDirect(
-                clientXmlTransportPtr(new clientXmlTransport_direct));
-            rpcPtr rpcSampleAddP("sample.add", paramListSampleAdd);
-            rpcSampleAddP->call(&clientDirect, &carriageParmDirect);
-            TEST(rpcSampleAddP->isFinished());
-            TEST(rpcSampleAddP->isSuccessful());
-            EXPECT_ERROR(fault fault0(rpcSampleAddP->getFault()););
-            value_int const resultDirect(rpcSampleAddP->getResult());
-            TEST(static_cast<int>(resultDirect) == 12);
-        }
-        {
-            /* Test with implicit RPC -- success */
-            rpcOutcome outcome;
-            clientDirect.call(&carriageParmDirect, "sample.add",
-                              paramListSampleAdd, &outcome);
-            TEST(outcome.succeeded());
-            value_int const result(outcome.getResult());
-            TEST(static_cast<int>(result) == 12);
-        }
-        {
-            /* Test with implicit RPC - failure */
-            rpcOutcome outcome;
-            clientDirect.call(&carriageParmDirect, "nosuchmethod",
-                              paramList(), &outcome);
-            TEST(!outcome.succeeded());
-            TEST(outcome.getFault().getCode() == fault::CODE_NO_SUCH_METHOD);
-            TEST(outcome.getFault().getDescription().size() > 0);
-        }
-
-        clientDirectAsyncTestSuite().run(indentation+1);
+#else
+        // This fails because there is no Curl transport in the library.
+        EXPECT_ERROR(clientXmlTransport_curl transportc0;);
+#endif
     }
 };
 
@@ -367,7 +451,7 @@ public:
     virtual string suiteName() {
         return "carriageParmTestSuite";
     }
-    virtual void runtests(unsigned int const indentation) {
+    virtual void runtests(unsigned int) {
         carriageParm_http0 carriageParm1("http://suckthis.com");
         carriageParm_curl0 carriageParm2("http://suckthis.com");
         carriageParm_libwww0 carriageParm3("http://suckthis.com");
@@ -403,48 +487,72 @@ public:
     virtual string suiteName() {
         return "clientRpcTestSuite";
     }
-    virtual void runtests(unsigned int const indentation) {
-#if MUST_BUILD_CURL_CLIENT
-        clientXmlTransport_curl transportc0;
-        client_xml client0(&transportc0);
-        carriageParm_http0 carriageParmHttp("http://suckthis.com");
-        carriageParm_curl0 carriageParmCurl("http://suckthis.com");
-        connection connection0(&client0, &carriageParmHttp);
+    virtual void runtests(unsigned int) {
+
+        registry myRegistry;
         
-        paramList paramList0;
-
-        rpcPtr rpc0P("blowme", paramList0);
-
-        // This fails because RPC has not been executed
-        EXPECT_ERROR(value result(rpc0P->getResult()););
-
-        // This fails because server doesn't exist
-        EXPECT_ERROR(rpc0P->call(&client0, &carriageParmCurl););
-
-        rpcPtr rpc1P("blowme", paramList0);
-        // This fails because server doesn't exist
-        EXPECT_ERROR(rpc1P->call(connection0););
-
-        rpcPtr rpc2P("blowme", paramList0);
-
-        rpc2P->start(&client0, &carriageParmCurl);
-
-        client0.finishAsync(timeout());
-
-        // This fails because the RPC failed because server doesn't exist
-        EXPECT_ERROR(value result(rpc2P->getResult()););
-
-        // This fails because the RPC has already been executed
-        EXPECT_ERROR(rpc2P->start(connection0););
-
-        rpcPtr rpc3P("blowme", paramList0);
-        rpc3P->start(connection0);
-
-        client0.finishAsync(timeout());
+        myRegistry.addMethod("sample.add", methodPtr(new sampleAddMethod));
         
-        // This fails because the RPC failed because server doesn't exist
-        EXPECT_ERROR(value result(rpc3P->getResult()););
-#endif
+        carriageParm_direct carriageParm0(&myRegistry);
+        clientXmlTransport_direct transportDirect;
+        client_xml client0(&transportDirect);
+        paramList paramListSampleAdd;
+        paramListSampleAdd.add(value_int(5));
+        paramListSampleAdd.add(value_int(7));
+        paramList paramListEmpty;
+
+        {
+            /* Test a successful RPC */
+            rpcPtr rpcSampleAddP("sample.add", paramListSampleAdd);
+            TEST(!rpcSampleAddP->isFinished());
+            // This fails because RPC has not been executed
+            EXPECT_ERROR(value result(rpcSampleAddP->getResult()););
+            
+            rpcSampleAddP->call(&client0, &carriageParm0);
+
+            TEST(rpcSampleAddP->isFinished());
+            TEST(rpcSampleAddP->isSuccessful());
+            value_int const resultDirect(rpcSampleAddP->getResult());
+            TEST(static_cast<int>(resultDirect) == 12);
+            // This fails because the RPC succeeded
+            EXPECT_ERROR(fault fault0(rpcSampleAddP->getFault()););
+            // This fails because the RPC has already been executed
+            EXPECT_ERROR(
+                rpcSampleAddP->call(&client0, &carriageParm0););
+            // This fails because the RPC has already been executed
+            EXPECT_ERROR(
+                rpcSampleAddP->start(&client0, &carriageParm0););
+        }
+        {
+            /* Test a failed RPC */
+            rpcPtr const rpcSampleAddP("sample.add", paramListEmpty);
+            rpcSampleAddP->call(&client0, &carriageParm0);
+            TEST(rpcSampleAddP->isFinished());
+            TEST(!rpcSampleAddP->isSuccessful());
+            fault const fault0(rpcSampleAddP->getFault());
+            TEST(fault0.getCode() == fault::CODE_TYPE);
+            // This fails because the RPC failed
+            EXPECT_ERROR(value result(rpcSampleAddP->getResult()););
+            // This fails because the RPC has already been executed
+            EXPECT_ERROR(
+                rpcSampleAddP->call(&client0, &carriageParm0););
+            // This fails because the RPC has already been executed
+            EXPECT_ERROR(
+                rpcSampleAddP->start(&client0, &carriageParm0););
+        }
+        {
+            /* Test with a connection */
+
+            connection connection0(&client0, &carriageParm0);
+
+            rpcPtr const rpcSampleAddP("sample.add", paramListSampleAdd);
+
+            rpcSampleAddP->call(connection0);
+
+            TEST(rpcSampleAddP->isFinished());
+            value_int const resultDirect(rpcSampleAddP->getResult());
+            TEST(static_cast<int>(resultDirect) == 12);
+        }
     }
 };
 
@@ -456,7 +564,7 @@ public:
     virtual string suiteName() {
         return "clientPtrTestSuite";
     }
-    virtual void runtests(unsigned int const indentation) {
+    virtual void runtests(unsigned int) {
         registry myRegistry;
         
         myRegistry.addMethod("sample.add", methodPtr(new sampleAddMethod));
@@ -488,7 +596,7 @@ public:
     virtual string suiteName() {
         return "serverAccessorTestSuite";
     }
-    virtual void runtests(unsigned int const indentation) {
+    virtual void runtests(unsigned int) {
         clientXmlTransportPtr const transportP(new clientXmlTransport_direct);
         clientPtr const clientP(new client_xml(transportP));
         registry myRegistry;
@@ -513,6 +621,7 @@ clientTestSuite::suiteName() {
 }
 
 
+
 void
 clientTestSuite::runtests(unsigned int const indentation) {
 
@@ -522,6 +631,8 @@ clientTestSuite::runtests(unsigned int const indentation) {
     
     carriageParmTestSuite().run(indentation+1);
     
+    clientCurlTestSuite().run(indentation+1);
+
     clientRpcTestSuite().run(indentation+1);
     
     clientPtrTestSuite().run(indentation+1);
