@@ -2,10 +2,20 @@
 
 #include "xmlrpc_config.h"
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <time.h>
+#include <fcntl.h>
+#ifdef _WIN32
+#  include <io.h>
+#else
+#  include <signal.h>
+#  include <sys/wait.h>
+#  include <grp.h>
+#endif
 
 #include "mallocvar.h"
 #include "xmlrpc-c/abyss.h"
@@ -13,6 +23,7 @@
 #include "xmlrpc-c/base.h"
 #include "xmlrpc-c/server.h"
 #include "xmlrpc-c/base_int.h"
+#include "xmlrpc-c/string_int.h"
 #include "xmlrpc-c/server_abyss.h"
 
 
@@ -33,18 +44,40 @@ static void die_if_fault_occurred(xmlrpc_env *env) {
 
 
 
-/*=========================================================================
-**  send_xml_data
-**=========================================================================
-**  Blast some XML data back to the client.
-*/
+static void
+addAuthCookie(xmlrpc_env * const envP,
+              TSession *   const abyssSessionP,
+              const char * const authCookie) {
+
+    const char * cookieResponse;
+    
+    xmlrpc_asprintf(&cookieResponse, "auth=%s", authCookie);
+    
+    if (cookieResponse == xmlrpc_strsol)
+        xmlrpc_faultf(envP, "Insufficient memory to generate cookie "
+                      "response header.");
+    else {
+        ResponseAddField(abyssSessionP, "Set-Cookie", cookieResponse);
+    
+        xmlrpc_strfree(cookieResponse);
+    }
+}   
+    
+
 
 static void 
 sendXmlData(xmlrpc_env * const envP,
-            TSession *   const r, 
-            char *       const buffer, 
-            size_t       const len) {
+            TSession *   const abyssSessionP, 
+            const char * const body, 
+            size_t       const len,
+            bool         const chunked) {
+/*----------------------------------------------------------------------------
+   Generate an HTTP response containing body 'body' of length 'len'
+   characters.
 
+   This is meant to run in the context of an Abyss URI handler for
+   Abyss session 'abyssSessionP'.
+-----------------------------------------------------------------------------*/
     const char * http_cookie = NULL;
         /* This used to set http_cookie to getenv("HTTP_COOKIE"), but
            that doesn't make any sense -- environment variables are not
@@ -52,56 +85,48 @@ sendXmlData(xmlrpc_env * const envP,
            - Bryan 2004.10.03.
         */
 
-    /* fwrite(buffer, sizeof(char), len, stderr); */
+    /* Various bugs before Xmlrpc-c 1.05 caused the response to be not
+       chunked in the most basic case, but chunked if the client explicitly
+       requested keepalive.  I think it's better not to chunk, because
+       it's simpler, so I removed this in 1.05.  I don't know what the
+       purpose of chunking would be, and an original comment suggests
+       the author wasn't sure chunking was a good idea.
 
-    /* XXX - Is it safe to chunk our response? */
-    ResponseChunked(r);
+       In 1.06 we added the user option to chunk.
+    */
+    if (chunked)
+        ResponseChunked(abyssSessionP);
 
-    ResponseStatus(r, 200);
-    
-    if (http_cookie) {
+    ResponseStatus(abyssSessionP, 200);
+
+    if (http_cookie)
         /* There's an auth cookie, so pass it back in the response. */
+        addAuthCookie(envP, abyssSessionP, http_cookie);
 
-        char *cookie_response;
- 
-        cookie_response = malloc(10+strlen(http_cookie));
-        sprintf(cookie_response, "auth=%s", http_cookie);
-        
-        /* Return abyss response. */
-        ResponseAddField(r, "Set-Cookie", cookie_response);
-
-        free(cookie_response);
-    }   
-    
     if ((size_t)(uint32_t)len != len)
-        xmlrpc_env_set_fault_formatted(
-            envP, XMLRPC_INTERNAL_ERROR,
-            "XML-RPC method generated a response too "
-            "large for Abyss to send");
+        xmlrpc_faultf(envP, "XML-RPC method generated a response too "
+                      "large for Abyss to send");
     else {
         uint32_t const abyssLen = (uint32_t)len;
-        ResponseContentType(r, "text/xml; charset=\"utf-8\"");
-        ResponseContentLength(r, abyssLen);
+
+        ResponseContentType(abyssSessionP, "text/xml; charset=\"utf-8\"");
+        ResponseContentLength(abyssSessionP, abyssLen);
         
-        ResponseWrite(r);
-        
-        HTTPWrite(r, buffer, abyssLen);
-        HTTPWriteEnd(r);
+        ResponseWriteStart(abyssSessionP);
+        ResponseWriteBody(abyssSessionP, body, abyssLen);
+        ResponseWriteEnd(abyssSessionP);
     }
 }
 
 
 
-/*=========================================================================
-**  send_error
-**=========================================================================
-**  Send an error back to the client.
-*/
-
 static void
-send_error(TSession *   const abyssSessionP, 
-           unsigned int const status) {
-
+sendError(TSession *   const abyssSessionP, 
+          unsigned int const status) {
+/*----------------------------------------------------------------------------
+  Send an error response back to the client.
+   
+-----------------------------------------------------------------------------*/
     ResponseStatus(abyssSessionP, (uint16_t) status);
     ResponseError(abyssSessionP);
 }
@@ -157,13 +182,13 @@ getBody(xmlrpc_env *        const envP,
 
     if (trace)
         fprintf(stderr, "XML-RPC handler processing body.  "
-                "Content Size = %u bytes\n", contentSize);
+                "Content Size = %u bytes\n", (unsigned)contentSize);
 
     body = xmlrpc_mem_block_new(envP, 0);
     if (!envP->fault_occurred) {
-        unsigned int bytesRead;
+        size_t bytesRead;
         const char * chunkPtr;
-        int chunkLen;
+        size_t chunkLen;
 
         bytesRead = 0;
 
@@ -172,6 +197,8 @@ getBody(xmlrpc_env *        const envP,
                                &chunkPtr, &chunkLen);
             bytesRead += chunkLen;
 
+            assert(bytesRead <= contentSize);
+
             XMLRPC_MEMBLOCK_APPEND(char, envP, body, chunkPtr, chunkLen);
             if (bytesRead < contentSize)
                 refillBufferFromConnection(envP, abyssSessionP, trace);
@@ -221,10 +248,26 @@ validateContentType(TSession *     const httpRequestP,
 -----------------------------------------------------------------------------*/
     const char * const content_type =
         RequestHeaderValue(httpRequestP, "content-type");
-    if (content_type == NULL || strcmp(content_type, "text/xml") != 0)
+
+    if (content_type == NULL)
         *httpErrorP = 400;
-    else
-        *httpErrorP = 0;
+    else {
+        const char * const sempos = strchr(content_type, ';');
+        unsigned int baselen;
+            /* Length of the base portion of the content type, e.g.
+               "text/xml" int "text/xml;charset=utf-8"
+            */
+
+        if (sempos)
+            baselen = sempos - content_type;
+        else
+            baselen = strlen(content_type);
+
+        if (!xmlrpc_strneq(content_type, "text/xml", baselen))
+            *httpErrorP = 400;
+        else
+            *httpErrorP = 0;
+    }
 }
 
 
@@ -277,7 +320,7 @@ traceHandlerCalled(TSession * const abyssSessionP) {
     const char * methodDesc;
     const TRequestInfo * requestInfoP;
 
-    fprintf(stderr, "xmlrpc_server_abyss RPC2 handler called.\n");
+    fprintf(stderr, "xmlrpc_server_abyss URI path handler called.\n");
 
     SessionGetRequestInfo(abyssSessionP, &requestInfoP);
 
@@ -309,6 +352,7 @@ static void
 processCall(TSession *        const abyssSessionP,
             size_t            const contentSize,
             xmlrpc_registry * const registryP,
+            bool              const wantChunk,
             const char *      const trace) {
 /*----------------------------------------------------------------------------
    Handle an RPC request.  This is an HTTP request that has the proper form
@@ -319,7 +363,8 @@ processCall(TSession *        const abyssSessionP,
     xmlrpc_env env;
 
     if (trace)
-        fprintf(stderr, "xmlrpc_server_abyss RPC2 handler processing RPC.\n");
+        fprintf(stderr,
+                "xmlrpc_server_abyss URI path handler processing RPC.\n");
 
     xmlrpc_env_init(&env);
 
@@ -342,7 +387,8 @@ processCall(TSession *        const abyssSessionP,
                 /* Send out the result. */
                 sendXmlData(&env, abyssSessionP, 
                             XMLRPC_MEMBLOCK_CONTENTS(char, output),
-                            XMLRPC_MEMBLOCK_SIZE(char, output));
+                            XMLRPC_MEMBLOCK_SIZE(char, output),
+                            wantChunk);
                 
                 XMLRPC_MEMBLOCK_FREE(char, output);
             }
@@ -351,9 +397,9 @@ processCall(TSession *        const abyssSessionP,
     }
     if (env.fault_occurred) {
         if (env.fault_code == XMLRPC_TIMEOUT_ERROR)
-            send_error(abyssSessionP, 408); /* 408 Request Timeout */
+            sendError(abyssSessionP, 408); /* 408 Request Timeout */
         else
-            send_error(abyssSessionP, 500); /* 500 Internal Server Error */
+            sendError(abyssSessionP, 500); /* 500 Internal Server Error */
     }
 
     xmlrpc_env_clean(&env);
@@ -375,19 +421,20 @@ struct uriHandlerXmlrpc {
    that is specific to the Xmlrpc-c handler.
 -----------------------------------------------------------------------------*/
     xmlrpc_registry * registryP;
-    const char *      filename;  /* malloc'ed */
+    const char *      uriPath;  /* malloc'ed */
+    bool              chunkResponse;
+        /* The handler should chunk its response whenever possible */
 };
 
 
 
 static void
-termUriHandler(URIHandler2 * const this) {
+termUriHandler(void * const arg) {
 
-    struct uriHandlerXmlrpc * const uriHandlerXmlrpcP = this->userdata;
+    struct uriHandlerXmlrpc * const uriHandlerXmlrpcP = arg;
 
-    free((void*)uriHandlerXmlrpcP->filename);
+    xmlrpc_strfree(uriHandlerXmlrpcP->uriPath);
     free(uriHandlerXmlrpcP);
-    free(this);
 }
 
 
@@ -421,8 +468,8 @@ handleXmlrpcReq(URIHandler2 * const this,
     /* Note that requestInfoP->uri is not the whole URI.  It is just
        the "file name" part of it.
     */
-    if (strcmp(requestInfoP->uri, uriHandlerXmlrpcP->filename) != 0)
-        /* It's for the filename (e.g. "/RPC2") that we're supposed to
+    if (strcmp(requestInfoP->uri, uriHandlerXmlrpcP->uriPath) != 0)
+        /* It's for the path (e.g. "/RPC2") that we're supposed to
            handle.
         */
         *handledP = FALSE;
@@ -433,17 +480,17 @@ handleXmlrpcReq(URIHandler2 * const this,
            "405 Method Not Allowed". 
         */
         if (requestInfoP->method != m_post)
-            send_error(abyssSessionP, 405);
+            sendError(abyssSessionP, 405);
         else {
             unsigned int httpError;
             storeCookies(abyssSessionP, &httpError);
             if (httpError)
-                send_error(abyssSessionP, httpError);
+                sendError(abyssSessionP, httpError);
             else {
                 unsigned int httpError;
                 validateContentType(abyssSessionP, &httpError);
                 if (httpError)
-                    send_error(abyssSessionP, httpError);
+                    sendError(abyssSessionP, httpError);
                 else {
                     unsigned int httpError;
                     size_t contentSize;
@@ -451,16 +498,18 @@ handleXmlrpcReq(URIHandler2 * const this,
                     processContentLength(abyssSessionP, 
                                          &contentSize, &httpError);
                     if (httpError)
-                        send_error(abyssSessionP, httpError);
+                        sendError(abyssSessionP, httpError);
                     else 
                         processCall(abyssSessionP, contentSize,
-                                    uriHandlerXmlrpcP->registryP, trace_abyss);
+                                    uriHandlerXmlrpcP->registryP,
+                                    uriHandlerXmlrpcP->chunkResponse,
+                                    trace_abyss);
                 }
             }
         }
     }
     if (trace_abyss)
-        fprintf(stderr, "xmlrpc_server_abyss RPC2 handler returning.\n");
+        fprintf(stderr, "xmlrpc_server_abyss URI path handler returning.\n");
 }
 
 
@@ -473,168 +522,109 @@ handleXmlrpcReq(URIHandler2 * const this,
 */
 
 static xmlrpc_bool 
-xmlrpc_server_abyss_default_handler (TSession * const r) {
+xmlrpc_server_abyss_default_handler(TSession * const sessionP) {
 
     if (trace_abyss)
         fprintf(stderr, "xmlrpc_server_abyss default handler called.\n");
 
-    send_error(r, 404);
+    sendError(sessionP, 404);
 
     return TRUE;
 }
 
 
 
-/**************************************************************************
-**
-** The code below was adapted from the main.c file of the Abyss webserver
-** project. In addition to the other copyrights on this file, the following
-** code is also under this copyright:
-**
-** Copyright (C) 2000 by Moez Mahfoudh <mmoez@bigfoot.com>.
-** All rights reserved.
-**
-** Redistribution and use in source and binary forms, with or without
-** modification, are permitted provided that the following conditions
-** are met:
-** 1. Redistributions of source code must retain the above copyright
-**    notice, this list of conditions and the following disclaimer.
-** 2. Redistributions in binary form must reproduce the above copyright
-**    notice, this list of conditions and the following disclaimer in the
-**    documentation and/or other materials provided with the distribution.
-** 3. The name of the author may not be used to endorse or promote products
-**    derived from this software without specific prior written permission.
-** 
-** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-** SUCH DAMAGE.
-**
-**************************************************************************/
-
-#include <time.h>
-#include <fcntl.h>
-
-#ifdef _WIN32
-#include <io.h>
-#else
-#include <signal.h>
-#include <sys/wait.h>
-#include <grp.h>
-#endif  /* _WIN32 */
-
-
-
 static void 
-sigterm(int const sig) {
-    TraceExit("Signal %d received. Exiting...\n",sig);
-}
-
-
-static void 
-sigchld(int const sig ATTR_UNUSED) {
+sigchld(int const signalClass ATTR_UNUSED) {
 /*----------------------------------------------------------------------------
    This is a signal handler for a SIGCHLD signal (which informs us that
    one of our child processes has terminated).
 
-   We respond by reaping the zombie process.
-
-   Implementation note: In some systems, just setting the signal handler
-   to SIG_IGN (ignore signal) does this.  In others, it doesn't.
+   The only child processes we have are those that belong to the Abyss
+   server (and then only if the Abyss server was configured to use
+   forking as a threading mechanism), so we respond by passing the
+   signal on to the Abyss server.
 -----------------------------------------------------------------------------*/
-#ifndef _WIN32
-    pid_t pid;
-    int status;
+#ifndef WIN32
+    bool childrenLeft;
+    bool error;
+
+    assert(signalClass == SIGCHLD);
+
+    error = false;
+    childrenLeft = true;  /* initial assumption */
     
     /* Reap defunct children until there aren't any more. */
-    for (;;) {
-        pid = waitpid( (pid_t) -1, &status, WNOHANG );
+    while (childrenLeft && !error) {
+        int status;
+        pid_t pid;
+
+        pid = waitpid((pid_t) -1, &status, WNOHANG);
     
-        /* none left */
-        if (pid==0)
-            break;
-    
-        if (pid<0) {
+        if (pid == 0)
+            childrenLeft = false;
+        else if (pid < 0) {
             /* because of ptrace */
-            if (errno==EINTR)   
-                continue;
-        
-            break;
-        }
+            if (errno != EINTR)   
+                error = true;
+        } else
+            ServerHandleSigchld(pid);
     }
-#endif /* _WIN32 */
+#endif /* WIN32 */
 }
 
-static TServer globalSrv;
-    /* When you use the old interface (xmlrpc_server_abyss_init(), etc.),
-       this is the Abyss server to which they refer.  Obviously, there can be
-       only one Abyss server per program using this interface.
-    */
 
-
-void 
-xmlrpc_server_abyss_init(int          const flags ATTR_UNUSED, 
-                         const char * const config_file) {
-
-    DateInit();
-    MIMETypeInit();
-
-    ServerCreate(&globalSrv, "XmlRpcServer", 8080, DEFAULT_DOCS, NULL);
-    
-    ConfReadServerFile(config_file, &globalSrv);
-
-    xmlrpc_server_abyss_init_registry();
-        /* Installs /RPC2 handler and default handler that use the
-           built-in registry.
-        */
-
-    ServerInit(&globalSrv);
-}
+struct signalHandlers {
+    struct sigaction pipe;
+    struct sigaction chld;
+};
 
 
 
 static void
-setupSignalHandlers(void) {
-#ifndef _WIN32
+setupSignalHandlers(struct signalHandlers * const oldHandlersP) {
+#ifndef WIN32
     struct sigaction mysigaction;
     
     sigemptyset(&mysigaction.sa_mask);
     mysigaction.sa_flags = 0;
 
-    /* These signals abort the program, with tracing */
-    mysigaction.sa_handler = sigterm;
-    sigaction(SIGTERM, &mysigaction, NULL);
-    sigaction(SIGINT,  &mysigaction, NULL);
-    sigaction(SIGHUP,  &mysigaction, NULL);
-    sigaction(SIGUSR1, &mysigaction, NULL);
-
     /* This signal indicates connection closed in the middle */
     mysigaction.sa_handler = SIG_IGN;
-    sigaction(SIGPIPE, &mysigaction, NULL);
+    sigaction(SIGPIPE, &mysigaction, &oldHandlersP->pipe);
     
     /* This signal indicates a child process (request handler) has died */
     mysigaction.sa_handler = sigchld;
-    sigaction(SIGCHLD, &mysigaction, NULL);
+    sigaction(SIGCHLD, &mysigaction, &oldHandlersP->chld);
 #endif
 }    
 
 
 
 static void
-runServerDaemon(TServer *  const srvP,
+restoreSignalHandlers(struct signalHandlers const oldHandlers) {
+#ifndef WIN32
+
+    sigaction(SIGPIPE, &oldHandlers.pipe, NULL);
+    sigaction(SIGCHLD, &oldHandlers.chld, NULL);
+
+#endif
+}
+
+
+
+static void
+runServerDaemon(TServer *  const serverP,
                 runfirstFn const runfirst,
                 void *     const runfirstArg) {
 
-    setupSignalHandlers();
+    struct signalHandlers oldHandlers;
 
-    ServerDaemonize(srvP);
+    setupSignalHandlers(&oldHandlers);
+
+    ServerUseSigchld(serverP);
+
+    ServerDaemonize(serverP);
     
     /* We run the user supplied runfirst after forking, but before accepting
        connections (helpful when running with threads)
@@ -642,26 +632,46 @@ runServerDaemon(TServer *  const srvP,
     if (runfirst)
         runfirst(runfirstArg);
 
-    ServerRun(srvP);
+    ServerRun(serverP);
 
-    /* We can't exist here because ServerRun doesn't return */
-    XMLRPC_ASSERT(FALSE);
+    restoreSignalHandlers(oldHandlers);
 }
 
 
 
-void 
-xmlrpc_server_abyss_run_first(runfirstFn const runfirst,
-                              void *     const runfirstArg) {
+static void
+setHandler(xmlrpc_env *      const envP,
+           TServer *         const srvP,
+           const char *      const uriPath,
+           xmlrpc_registry * const registryP,
+           bool              const chunkResponse) {
     
-    runServerDaemon(&globalSrv, runfirst, runfirstArg);
-}
+    struct uriHandlerXmlrpc * uriHandlerXmlrpcP;
+    URIHandler2 uriHandler;
+    abyss_bool success;
 
+    trace_abyss = getenv("XMLRPC_TRACE_ABYSS");
+                                 
+    MALLOCVAR_NOFAIL(uriHandlerXmlrpcP);
 
+    uriHandlerXmlrpcP->registryP     = registryP;
+    uriHandlerXmlrpcP->uriPath       = strdup(uriPath);
+    uriHandlerXmlrpcP->chunkResponse = chunkResponse;
 
-void 
-xmlrpc_server_abyss_run(void) {
-    runServerDaemon(&globalSrv, NULL, NULL);
+    uriHandler.handleReq2 = handleXmlrpcReq;
+    uriHandler.handleReq1 = NULL;
+    uriHandler.userdata   = uriHandlerXmlrpcP;
+    uriHandler.init       = NULL;
+    uriHandler.term       = &termUriHandler;
+
+    ServerAddHandler2(srvP, &uriHandler, &success);
+
+    if (!success)
+        xmlrpc_faultf(envP, "Abyss failed to register the Xmlrpc-c request "
+                      "handler.  ServerAddHandler2() failed.");
+
+    if (envP->fault_occurred)
+        free(uriHandlerXmlrpcP);
 }
 
 
@@ -669,34 +679,44 @@ xmlrpc_server_abyss_run(void) {
 void
 xmlrpc_server_abyss_set_handler(xmlrpc_env *      const envP,
                                 TServer *         const srvP,
-                                const char *      const filename,
+                                const char *      const uriPath,
                                 xmlrpc_registry * const registryP) {
+
+    setHandler(envP, srvP, uriPath, registryP, false);
+}
+
     
-    struct uriHandlerXmlrpc * uriHandlerXmlrpcP;
-    URIHandler2 * uriHandlerP;
-    abyss_bool success;
+
+static void
+setHandlers(TServer *         const srvP,
+            const char *      const uriPath,
+            xmlrpc_registry * const registryP,
+            bool              const chunkResponse) {
+
+    xmlrpc_env env;
+
+    xmlrpc_env_init(&env);
 
     trace_abyss = getenv("XMLRPC_TRACE_ABYSS");
                                  
-    MALLOCVAR_NOFAIL(uriHandlerP);
+    setHandler(&env, srvP, uriPath, registryP, chunkResponse);
+    
+    if (env.fault_occurred)
+        abort();
 
-    uriHandlerP->handleReq2 = handleXmlrpcReq;
-    uriHandlerP->handleReq1 = NULL;
-    uriHandlerP->init       = NULL;
-    uriHandlerP->term       = &termUriHandler;
+    ServerDefaultHandler(srvP, xmlrpc_server_abyss_default_handler);
 
-    MALLOCVAR_NOFAIL(uriHandlerXmlrpcP);
+    xmlrpc_env_clean(&env);
+}
 
-    uriHandlerXmlrpcP->registryP = registryP;
-    uriHandlerXmlrpcP->filename  = strdup(filename);
 
-    uriHandlerP->userdata = uriHandlerXmlrpcP;
 
-    ServerAddHandler2(srvP, uriHandlerP, &success);
+void
+xmlrpc_server_abyss_set_handlers2(TServer *         const srvP,
+                                  const char *      const uriPath,
+                                  xmlrpc_registry * const registryP) {
 
-    if (!success)
-        xmlrpc_faultf(envP, "Abyss failed to register the Xmlrpc-c request "
-                      "handler.  ServerAddHandler2() failed.");
+    setHandlers(srvP, uriPath, registryP, false);
 }
 
 
@@ -705,24 +725,13 @@ void
 xmlrpc_server_abyss_set_handlers(TServer *         const srvP,
                                  xmlrpc_registry * const registryP) {
 
-    xmlrpc_env env;
-
-    xmlrpc_env_init(&env);
-
-    trace_abyss = getenv("XMLRPC_TRACE_ABYSS");
-                                 
-    xmlrpc_server_abyss_set_handler(&env, srvP, "/RPC2", registryP);
-    
-    if (env.fault_occurred)
-        abort();
-
-    ServerDefaultHandler(srvP, xmlrpc_server_abyss_default_handler);
+    setHandlers(srvP, "/RPC2", registryP, false);
 }
 
 
 
 static void
-oldHighLevelAbyssRun(xmlrpc_env *                      const envP ATTR_UNUSED,
+oldHighLevelAbyssRun(xmlrpc_env *                      const envP,
                      const xmlrpc_server_abyss_parms * const parmsP,
                      unsigned int                      const parmSize) {
 /*----------------------------------------------------------------------------
@@ -736,29 +745,35 @@ oldHighLevelAbyssRun(xmlrpc_env *                      const envP ATTR_UNUSED,
    Abyss parameters in memory.  That's a more conventional and
    flexible API.
 -----------------------------------------------------------------------------*/
-    TServer srv;
+    TServer server;
     runfirstFn runfirst;
     void * runfirstArg;
+    abyss_bool success;
     
     DateInit();
-    MIMETypeInit();
     
-    ServerCreate(&srv, "XmlRpcServer", 8080, DEFAULT_DOCS, NULL);
-    
-    ConfReadServerFile(parmsP->config_file_name, &srv);
+    success = ServerCreate(&server, "XmlRpcServer", 8080, DEFAULT_DOCS, NULL);
+
+    if (!success)
+        xmlrpc_faultf(envP, "Failed to create Abyss server object");
+    else {
+        ConfReadServerFile(parmsP->config_file_name, &server);
         
-    xmlrpc_server_abyss_set_handlers(&srv, parmsP->registryP);
+        setHandlers(&server, "/RPC2", parmsP->registryP, false);
         
-    ServerInit(&srv);
+        ServerInit(&server);
     
-    if (parmSize >= XMLRPC_APSIZE(runfirst_arg)) {
-        runfirst    = parmsP->runfirst;
-        runfirstArg = parmsP->runfirst_arg;
-    } else {
-        runfirst    = NULL;
-        runfirstArg = NULL;
+        if (parmSize >= XMLRPC_APSIZE(runfirst_arg)) {
+            runfirst    = parmsP->runfirst;
+            runfirstArg = parmsP->runfirst_arg;
+        } else {
+            runfirst    = NULL;
+            runfirstArg = NULL;
+        }
+        runServerDaemon(&server, runfirst, runfirstArg);
+
+        ServerFree(&server);
     }
-    runServerDaemon(&srv, runfirst, runfirstArg);
 }
 
 
@@ -795,7 +810,7 @@ extractServerCreateParms(
     unsigned int                      const parmSize,
     abyss_bool *                      const socketBoundP,
     unsigned int *                    const portNumberP,
-    TSocket *                         const socketFdP,
+    TOsSocket *                       const socketFdP,
     const char **                     const logFileNameP) {
                    
 
@@ -835,45 +850,179 @@ extractServerCreateParms(
 
 
 static void
-normalLevelAbyssRun(xmlrpc_env *                      const envP,
-                    const xmlrpc_server_abyss_parms * const parmsP,
-                    unsigned int                      const parmSize) {
+createServerBoundSocket(xmlrpc_env * const envP,
+                        TOsSocket    const socketFd,
+                        const char * const logFileName,
+                        TServer *    const serverP,
+                        TSocket **   const socketPP) {
+
+    TSocket * socketP;
+    const char * error;
     
+    SocketUnixCreateFd(socketFd, &socketP);
+    
+    if (!socketP)
+        xmlrpc_faultf(envP, "Unable to create Abyss socket out of "
+                      "file descriptor %d.", socketFd);
+    else {
+        ServerCreateSocket2(serverP, socketP, &error);
+        if (error) {
+            xmlrpc_faultf(envP, "Abyss failed to create server.  %s",
+                          error);
+            xmlrpc_strfree(error);
+        } else {
+            *socketPP = socketP;
+                    
+            ServerSetName(serverP, "XmlRpcServer");
+            
+            if (logFileName)
+                ServerSetLogFileName(serverP, logFileName);
+        }
+        if (envP->fault_occurred)
+                    SocketDestroy(socketP);
+    }
+}
+
+
+
+static void
+createServer(xmlrpc_env *                      const envP,
+             const xmlrpc_server_abyss_parms * const parmsP,
+             unsigned int                      const parmSize,
+             TServer *                         const serverP,
+             TSocket **                        const socketPP) {
+/*----------------------------------------------------------------------------
+   Create a bare server.  It will need further setup before it is ready
+   to use.
+-----------------------------------------------------------------------------*/
     abyss_bool socketBound;
     unsigned int portNumber;
-    TSocket socketFd;
+    TOsSocket socketFd;
     const char * logFileName;
-    
-    DateInit();
-    MIMETypeInit();
 
     extractServerCreateParms(envP, parmsP, parmSize,
                              &socketBound, &portNumber, &socketFd,
                              &logFileName);
 
     if (!envP->fault_occurred) {
-        TServer server;
-
         if (socketBound)
-            ServerCreateSocket(&server, "XmlRpcServer", socketFd,
-                               DEFAULT_DOCS, logFileName);
-        else
-            ServerCreate(&server, "XmlRpcServer", portNumber, DEFAULT_DOCS, 
-                         logFileName);
+            createServerBoundSocket(envP, socketFd, logFileName,
+                                    serverP, socketPP);
+        else {
+            abyss_bool success;
+
+            success = ServerCreate(serverP, "XmlRpcServer", portNumber,
+                                   DEFAULT_DOCS, logFileName);
+
+            if (!success)
+                xmlrpc_faultf(envP, "Failed to create Abyss server object");
+            
+            *socketPP = NULL;
+        }
+        if (logFileName)
+            xmlrpc_strfree(logFileName);
+    }
+}
+
+
+
+static bool
+chunkResponseParm(const xmlrpc_server_abyss_parms * const parmsP,
+                  unsigned int                      const parmSize) {
+
+    return
+        parmSize >= XMLRPC_APSIZE(chunk_response) &&
+        parmsP->chunk_response;
+}    
+
+
+
+static const char *
+uriPathParm(const xmlrpc_server_abyss_parms * const parmsP,
+            unsigned int                      const parmSize) {
+    
+    const char * uriPath;
+
+    if (parmSize >= XMLRPC_APSIZE(uri_path) && parmsP->uri_path)
+        uriPath = parmsP->uri_path;
+    else
+        uriPath = "/RPC2";
+
+    return uriPath;
+}
+
+
+
+static xmlrpc_server_shutdown_fn shutdownAbyss;
+
+static void
+shutdownAbyss(xmlrpc_env * const envP,
+              void *       const context,
+              const char * const comment ATTR_UNUSED) {
+/*----------------------------------------------------------------------------
+   Tell Abyss to wrap up whatever it's doing and shut down.
+
+   This is a server shutdown function to be registered in the method
+   registry, for use by the 'system.shutdown' system method.
+
+   After we return, Abyss will finish up the system.shutdown and any
+   other connections that are in progress, then the call to
+   ServerRun() etc. will return.  But Abyss may be stuck waiting for
+   something, such as the next HTTP connection.  In that case, until it
+   gets what it's waiting for, it won't even know it's supposed t shut
+   down.  In particular, a caller of system.shutdown may have to execute
+   one more RPC in order for the shutdown to happen.
+-----------------------------------------------------------------------------*/
+    TServer * const serverP = context;
+
+    xmlrpc_env_init(envP);
+    
+    ServerTerminate(serverP);
+}
+
+
+
+static void
+normalLevelAbyssRun(xmlrpc_env *                      const envP,
+                    const xmlrpc_server_abyss_parms * const parmsP,
+                    unsigned int                      const parmSize) {
+    
+    TServer server;
+    TSocket * socketP;
+
+    DateInit();
+
+    createServer(envP, parmsP, parmSize, &server, &socketP);
+
+    if (!envP->fault_occurred) {
+        struct signalHandlers oldHandlers;
 
         setAdditionalServerParms(parmsP, parmSize, &server);
 
-        xmlrpc_server_abyss_set_handlers(&server, parmsP->registryP);
-        
+        setHandlers(&server, uriPathParm(parmsP, parmSize), parmsP->registryP,
+                    chunkResponseParm(parmsP, parmSize));
+
         ServerInit(&server);
         
-        setupSignalHandlers();
+        setupSignalHandlers(&oldHandlers);
+
+        ServerUseSigchld(&server);
+        
+        if (0)
+            /* Too much of a security risk.  In 1.07, there is a server
+               parameter to enable this.
+            */
+            xmlrpc_registry_set_shutdown(parmsP->registryP,
+                                         &shutdownAbyss, &server);
         
         ServerRun(&server);
 
-        /* We can't exist here because ServerRun doesn't return */
-        XMLRPC_ASSERT(FALSE);
-        xmlrpc_strfree(logFileName);
+        restoreSignalHandlers(oldHandlers);
+
+        ServerFree(&server);
+
+        if (socketP)
+            SocketDestroy(socketP);
     }
 }
 
@@ -890,7 +1039,7 @@ xmlrpc_server_abyss(xmlrpc_env *                      const envP,
         xmlrpc_faultf(envP,
                       "You must specify members at least up through "
                       "'registryP' in the server parameters argument.  "
-                      "That would mean the parameter size would be >= %u "
+                      "That would mean the parameter size would be >= %lu "
                       "but you specified a size of %u",
                       XMLRPC_APSIZE(registryP), parmSize);
     else {
@@ -904,14 +1053,25 @@ xmlrpc_server_abyss(xmlrpc_env *                      const envP,
 
 
 /*=========================================================================
-**  XML-RPC Server Method Registry
-**=========================================================================
-**  A simple front-end to our method registry.
+  XML-RPC Server Method Registry
+
+  This is an old deprecated form of the server facilities that uses
+  global variables.
+=========================================================================*/
+
+/* These global variables must be treated as read-only after the
+   server has started.
 */
 
-/* XXX - This variable is *not* currently threadsafe. Once the server has
-** been started, it must be treated as read-only. */
-static xmlrpc_registry *builtin_registryP;
+static TServer globalSrv;
+    /* When you use the old interface (xmlrpc_server_abyss_init(), etc.),
+       this is the Abyss server to which they refer.  Obviously, there can be
+       only one Abyss server per program using this interface.
+    */
+
+static xmlrpc_registry * builtin_registryP;
+
+
 
 void 
 xmlrpc_server_abyss_init_registry(void) {
@@ -932,7 +1092,7 @@ xmlrpc_server_abyss_init_registry(void) {
     die_if_fault_occurred(&env);
     xmlrpc_env_clean(&env);
 
-    xmlrpc_server_abyss_set_handlers(&globalSrv, builtin_registryP);
+    setHandlers(&globalSrv, "/RPC2", builtin_registryP, false);
 }
 
 
@@ -951,9 +1111,9 @@ xmlrpc_server_abyss_registry(void) {
 
 /* A quick & easy shorthand for adding a method. */
 void 
-xmlrpc_server_abyss_add_method (char *        const method_name,
-                                xmlrpc_method const method,
-                                void *        const user_data) {
+xmlrpc_server_abyss_add_method(char *        const method_name,
+                               xmlrpc_method const method,
+                               void *        const user_data) {
     xmlrpc_env env;
 
     xmlrpc_env_init(&env);
@@ -966,11 +1126,11 @@ xmlrpc_server_abyss_add_method (char *        const method_name,
 
 
 void
-xmlrpc_server_abyss_add_method_w_doc (char *        const method_name,
-                                      xmlrpc_method const method,
-                                      void *        const user_data,
-                                      char *        const signature,
-                                      char *        const help) {
+xmlrpc_server_abyss_add_method_w_doc(char *        const method_name,
+                                     xmlrpc_method const method,
+                                     void *        const user_data,
+                                     char *        const signature,
+                                     char *        const help) {
 
     xmlrpc_env env;
     xmlrpc_env_init(&env);
@@ -983,6 +1143,50 @@ xmlrpc_server_abyss_add_method_w_doc (char *        const method_name,
 
 
 
+void 
+xmlrpc_server_abyss_init(int          const flags ATTR_UNUSED, 
+                         const char * const config_file) {
+
+    abyss_bool success;
+
+    DateInit();
+    MIMETypeInit();
+
+    success = ServerCreate(&globalSrv, "XmlRpcServer", 8080, DEFAULT_DOCS,
+                           NULL);
+    
+    if (!success)
+        abort();
+    else {
+        ConfReadServerFile(config_file, &globalSrv);
+
+        xmlrpc_server_abyss_init_registry();
+        /* Installs /RPC2 handler and default handler that use the
+           built-in registry.
+        */
+
+        ServerInit(&globalSrv);
+    }
+}
+
+
+
+void 
+xmlrpc_server_abyss_run_first(runfirstFn const runfirst,
+                              void *     const runfirstArg) {
+    
+    runServerDaemon(&globalSrv, runfirst, runfirstArg);
+}
+
+
+
+void 
+xmlrpc_server_abyss_run(void) {
+    runServerDaemon(&globalSrv, NULL, NULL);
+}
+
+
+
 /*
 ** Copyright (C) 2001 by First Peer, Inc. All rights reserved.
 **
